/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * **/
/*                                                                        */
/*      Flash12 - Analog read Demonstration Program for C language        */
/*                                                                        */
/* ---------------------------------------------------------------------- */
/* NOTE:                                                                  */ 
/*         This example is intended for users of Flash12 Card to gain     */
/*      fast mode of analog reading with trigger either in OFF, NORMAL    */
/*      or AUTO mode.                                                     */
/*         The example will acquire data samples from Channel 1 only.     */
/*      The gain range will be set to +-5 Volts with a delay of           */
/*      10 Micro seconds. If trigger in NORMAL or AUTO, a 50 samples      */
/*      of pre-trigger is required.                                       */
/*         A chart will show the analog input which is done with          */
/*      *Borland Turbo C. All the graphic routines will be done in        */
/*      GRAPH12.C. A triangle waveform is best supported by the example.  */
/*	   This program is compiled and linked with *Borland Turbo C      */
/*      Small Memory Model:                                               */
/*            Use SH_CALL() and link with the module MS_CALL.OBJ          */
/*      Large/Huge Memory Model:                                          */
/*            Change SH_CALL() to CCALL() and link with MS_CALL.OBJ       */
/*      MKFLASH.BAT is a batch file to do the compile                     */
/*      FLASH12.LNK is a link file to do the linking                      */
/*	   To run FLASH12.EXE, you have to use GDRV.COM V1.1.1,           */
/*      HSOVL.EXE V1.1.1 and GFIND.EXE V1.1. Load the above drivers       */
/*      as shown in the FLASH12 users guide. Include in the same          */ 
/*      directory, you should have *Borland's EGAVGA.BGI if you have a    */
/*      EGA/VGA adaptor or HERC.BGI if you have a HERC adaptor.	          */	
/*         A direct convention for translating "longs" or 32 bits         */
/*	integers to 2 consecutive "shorts/16 bits integers" in an array   */
/*	and vise versa is used in serveral places, they are :		  */
/*		shortints[0] = longint % 65536L				  */
/*		shortints[1] = longint / 65536L				  */
/*	    and								  */
/*		longint = shortints[1] * 65536L				  */ 
/*		longint += shortints[0]				          */
/*									  */
/*  *Borland is a trademark of Borland International, Inc.                */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * **/
#include <stdio.h>
#include "flash12.h"

#define	BUFFERLIMIT	640L			/* max size for array B */

static unsigned short 	A_Digital[BUFFERLIMIT];	/* short array A */
static unsigned short 	B_Digital[BUFFERLIMIT];	/* short array B */
static float            AnalogBuf[BUFFERLIMIT]; /* float array B */
static unsigned long    MaxSamples;             /* on board memory size */


/*****
	initialize and ready for Flash12 fast mode call
	this example use :
	1. set to read 1 channel
	2. set to read channel 1
	3. set a gain range of +-5 Volts
	4. set a delay of 10 micro seconds
*****/
static void
init_setup()
{
	/*** set number of channels to be read ***/
	A_Digital[0] = 1;			/* set 1 channel */		
	SH_CALL("N", AnalogBuf, A_Digital);

	/*** set channel order to be read ***/
	A_Digital[0] = 1;			/* read channel 1 */		
	SH_CALL("p", AnalogBuf, A_Digital);

	/*** set gain range ***/
	A_Digital[0] = 8;			/* range of +- 5 Volts */		
	SH_CALL("r", AnalogBuf, A_Digital);

	/*** set delay ***/
	AnalogBuf[0] = 10.00;		/* a delay of 10 micro seconds */	
	SH_CALL("j", AnalogBuf, A_Digital);
}


/*****
	get extended buffering conditions 
*****/
static
getBufferStatus()
{
	A_Digital[0] = 1;			
	SH_CALL("G", B_Digital, A_Digital);
	MaxSamples = B_Digital[3] * 65536L;	/* ram size on board */
	MaxSamples += B_Digital[2];
	if (MaxSamples > 0)
		return 1;
	else
	{
		printf("\nPlease check drivers and Flash12 card installation.");
		return 0;
	}
}



/***** 
	set high speed extended buffering parameters
*****/
static void
setControlParms(use_extended_buffer,reset,retrigger,triggered)
short use_extended_buffer;	/* turn extended buffering mode on */
short reset;			/* trigger reset */
short retrigger;		/* re-trigger */
short triggered;		/* triggered */
{
	A_Digital[0] = 0;
	B_Digital[0] = use_extended_buffer;
	B_Digital[1] = reset;
	B_Digital[2] = retrigger;
	B_Digital[3] = triggered;
	SH_CALL("P", B_Digital, A_Digital);
	
}

/***** 
	set high speed extended buffering parameters
*****/
static void
setExtendedParms(samples_requested)
unsigned long samples_requested;		/* samples to be acquired */
{
	A_Digital[0] = 1;		
	B_Digital[0] = (short) (samples_requested % 65536L);
	B_Digital[1] = (short) (samples_requested / 65536L);
	SH_CALL("P", B_Digital, A_Digital);
}


/***** 
	set high speed trigger parameters 
*****/
static void
setTriggerParms(trig_mode,trig_level,trig_timeout,pre_trig)
short trig_mode; 			/* Mode & Source */
short trig_level;			/* trigger level */
long trig_timeout;			/* trigger timeout */
long pre_trig;				/* pre-trigger count */
{
	A_Digital[0] = 2;
	B_Digital[0] = trig_mode;
	B_Digital[1] = trig_level;
	B_Digital[2] = (short) (trig_timeout % 65536L);
	B_Digital[3] = (short) (trig_timeout / 65536L);
	B_Digital[4] = (short) (pre_trig % 65536L);
	B_Digital[5] = (short) (pre_trig / 65536L);
	SH_CALL("P", B_Digital, A_Digital);
}


/*****
	Synchronized analog read 'H' for fast mode
*****/
static void
acquire_samples(samples)	/* amount of samples to read */
unsigned long samples;
{
	A_Digital[0] = (short) (samples % 65536L);
	A_Digital[1] = (short) (samples / 65536L);
	SH_CALL("H", AnalogBuf, A_Digital);
}


/*****
	'f' key was hitted
	skip forward 65536 samples from current point
*****/
static void
forward64K(samples_notCollected,hiword,loword)
unsigned long *samples_notCollected;
unsigned long *hiword,*loword;
{
	short i;

	if (*samples_notCollected >= 65536L)
	{
		for (i=0; i < (short)(65536L/BUFFERLIMIT); i++)	
			acquire_samples(BUFFERLIMIT);
		acquire_samples((long)(65536L%BUFFERLIMIT));
		*samples_notCollected -= 65536L;
		*hiword = *samples_notCollected / BUFFERLIMIT;  /* new hi */
		*loword = *samples_notCollected % BUFFERLIMIT;  /* new low */
	}
}


/*****
	all setControlParms,setExtendedParms and setTriggerParms have to be 
done before achieving fast mode, return TRUE if trigger is ON 
*****/
static
preFlashSet( trigmode, samples_requested )
int trigmode;
unsigned long samples_requested;
{
	setExtendedParms(samples_requested);
	switch (trigmode)
	{
		case TRIG_NORM:
			setControlParms(CRTL_EXTBUF_ON,CRTL_RESET_OFF,
					CRTL_RETRIG_ON,CRTL_TRIG_ON);
			setTriggerParms(TRIG_ANA_POS | TRIG_NORM,
				     	TRIG_LVL,TRIG_TIMEOUT,PRE_TRIG);
			return TRUE;
		case TRIG_AUTO:
			setControlParms(CRTL_EXTBUF_ON,CRTL_RESET_OFF,
					CRTL_RETRIG_ON,CRTL_TRIG_ON);
			setTriggerParms(TRIG_ANA_POS | TRIG_AUTO,
				     	TRIG_LVL,TRIG_TIMEOUT,PRE_TRIG);
			return TRUE;
		case TRIG_OFF:
		default :		/* default to trigger off */
			setControlParms(CRTL_EXTBUF_ON,CRTL_RESET_OFF,
					CRTL_RETRIG_ON,CRTL_TRIG_OFF);
			setTriggerParms(TRIG_ANA_POS | TRIG_OFF,
				     	TRIG_LVL,TRIG_TIMEOUT,PRE_TRIG);
			return FALSE;
	}
}


/*****
	executive the input key
	'n' 	  - go to next screen of samples
	'f'	  - skip forward 64K of samples
	space bar - get next set of data
	'Esc'	  - exit
*****/
static void
exec_inkey(inkey,is_triggered,dodrawing,drawtrigger,
			samples_notCollected,hiword,loword)
int inkey,is_triggered;
int *drawtrigger,*dodrawing;
unsigned long *samples_notCollected,*hiword,*loword;
{

	switch (inkey)
	{
		case ESC:		/* Esc key */
		case NXTSAMPLE:		/* space bar hitted */
			*dodrawing = FALSE;
			*drawtrigger = is_triggered;
			setControlParms(CRTL_EXTBUF_ON,
					CRTL_RESET_ON,
					CRTL_RETRIG_ON,
					CRTL_TRIG_OFF);
			*hiword = *loword = 0L;
			break;
		case NXTFORWARD:	/* 'f' key hitted */
			forward64K(samples_notCollected,
					hiword,loword);
		default :		/* next screen */
			*drawtrigger = FALSE;
			break;
	}
}

/*****
	fast read samples, depend on the size of the AnalogBuf(B) buffer,
	in this example, it is BUFFERLIMIT, it will call acquire_samples
	hiword + 1 times if loword > 0 or
	hiword times if loword == 0
*****/
static
collectSample(samples_requested, is_triggered)
unsigned long samples_requested;
int is_triggered;
{
	unsigned long hiword;		/* #multiples of BUFFERLIMIT */
	unsigned long loword;		/* fraction of BUFFERLIMIT */
	unsigned long samples_notCollected;   /* samples left to be collected */
	int dodrawing;			/* draw the graph */
	int drawtrigger,inkey='1';	/* set inkey != ESC */

	dodrawing = TRUE;
	drawtrigger = is_triggered;
	samples_notCollected = samples_requested; 
	hiword = samples_requested / BUFFERLIMIT;
	loword = samples_requested % BUFFERLIMIT;
	while (hiword--) 		/* loop hiword times */
	{	
		acquire_samples(BUFFERLIMIT);	/* read samples */
		samples_notCollected -= BUFFERLIMIT;
		if (dodrawing)		/* graphic : do drawing */
		{
			inkey = DrawAnalog(AnalogBuf,BUFFERLIMIT,
					drawtrigger,TRUE,samples_notCollected);
			exec_inkey(inkey,is_triggered,&dodrawing,&drawtrigger,
				&samples_notCollected,&hiword,&loword);
		}
	}					/* hiword is done */
	if (loword > 0L)	/* do the BUFFERLIMIT fraction */
	{
		acquire_samples(loword);
		if (dodrawing) 
			inkey = DrawAnalog(AnalogBuf,loword,
					drawtrigger,FALSE,0);
	}
	return inkey;
}

/*****
	flash analog read
	depend on trigger mode and size of samples to be read
	it will collect and display samples until users enter 'Esc' key
*****/
static void
flash12()
{
	int i,inkey;
	int is_triggered,trigmode;
	unsigned long samples_requested;/* required samples by user */

	do 				/* new sample and new trigmode */
	{
		SetTextMode();		/* graphic: to text mode */
		samples_requested = MaxSamples;
		trigmode = GetUserRequest(&samples_requested);
		if (trigmode == ESC)
			break;		/* user exit */
		is_triggered = preFlashSet( trigmode, samples_requested );
		SetGraphicMode();	/* graph: turn on graphic mode */
		do 			/* new samples of same trigmode */
		{
			inkey = collectSample(samples_requested, is_triggered);
		} while (inkey != ESC);
	} while (inkey == ESC);
	setControlParms(CRTL_EXTBUF_ON,	CRTL_RESET_ON,	/* reset driver */
			CRTL_RETRIG_ON,	CRTL_TRIG_OFF); /* to original state */
	SetTextMode();		/* graphic: to text mode */

}



/*****
	main program
*****/
void
main()
{
	if (getBufferStatus() == 1)
	{
		init_setup();
		Initgraphic();
		flash12();
	}
}


